@@ -9,11 +9,11 @@ import pytest
 from sklearn.utils.testing import assert_raises
 from sklearn.utils.testing import assert_raise_message
 from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_false
 from sklearn.utils.testing import assert_dict_equal
 from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_allclose_dense_sparse
+from sklearn.utils.testing import assert_almost_equal
 
 from sklearn.base import BaseEstimator
 from sklearn.externals import six
@@ -227,7 +227,7 @@ def test_column_transformer_dataframe():
             return self
 
         def transform(self, X, y=None):
-            assert_true(isinstance(X, (pd.DataFrame, pd.Series)))
+            assert isinstance(X, (pd.DataFrame, pd.Series))
             if isinstance(X, pd.Series):
                 X = X.to_frame()
             return X
@@ -309,7 +309,7 @@ def test_column_transformer_sparse_array():
             ct = ColumnTransformer([('trans', Trans(), col)],
                                    remainder=remainder,
                                    sparse_threshold=0.8)
-            assert_true(sparse.issparse(ct.fit_transform(X_sparse)))
+            assert sparse.issparse(ct.fit_transform(X_sparse))
             assert_allclose_dense_sparse(ct.fit_transform(X_sparse), res)
             assert_allclose_dense_sparse(ct.fit(X_sparse).transform(X_sparse),
                                          res)
@@ -317,7 +317,7 @@ def test_column_transformer_sparse_array():
     for col in [[0, 1], slice(0, 2)]:
         ct = ColumnTransformer([('trans', Trans(), col)],
                                sparse_threshold=0.8)
-        assert_true(sparse.issparse(ct.fit_transform(X_sparse)))
+        assert sparse.issparse(ct.fit_transform(X_sparse))
         assert_allclose_dense_sparse(ct.fit_transform(X_sparse), X_res_both)
         assert_allclose_dense_sparse(ct.fit(X_sparse).transform(X_sparse),
                                      X_res_both)
@@ -352,7 +352,7 @@ def test_column_transformer_sparse_stacking():
                                   sparse_threshold=0.8)
     col_trans.fit(X_array)
     X_trans = col_trans.transform(X_array)
-    assert_true(sparse.issparse(X_trans))
+    assert sparse.issparse(X_trans)
     assert_equal(X_trans.shape, (X_trans.shape[0], X_trans.shape[0] + 1))
     assert_array_equal(X_trans.toarray()[:, 1:], np.eye(X_trans.shape[0]))
     assert len(col_trans.transformers_) == 2
@@ -368,17 +368,47 @@ def test_column_transformer_sparse_stacking():
     assert_array_equal(X_trans[:, 1:], np.eye(X_trans.shape[0]))
 
 
+def test_column_transformer_mixed_cols_sparse():
+    df = np.array([['a', 1, True],
+                   ['b', 2, False]],
+                  dtype='O')
+
+    ct = make_column_transformer(
+        (OneHotEncoder(), [0]),
+        ('passthrough', [1, 2]),
+        sparse_threshold=1.0
+    )
+
+    # this shouldn't fail, since boolean can be coerced into a numeric
+    # See: https://github.com/scikit-learn/scikit-learn/issues/11912
+    X_trans = ct.fit_transform(df)
+    assert X_trans.getformat() == 'csr'
+    assert_array_equal(X_trans.toarray(), np.array([[1, 0, 1, 1],
+                                                    [0, 1, 2, 0]]))
+
+    ct = make_column_transformer(
+        (OneHotEncoder(), [0]),
+        ('passthrough', [0]),
+        sparse_threshold=1.0
+    )
+    with pytest.raises(ValueError,
+                       match="For a sparse output, all columns should"):
+        # this fails since strings `a` and `b` cannot be
+        # coerced into a numeric.
+        ct.fit_transform(df)
+
+
 def test_column_transformer_sparse_threshold():
     X_array = np.array([['a', 'b'], ['A', 'B']], dtype=object).T
     # above data has sparsity of 4 / 8 = 0.5
 
-    # if all sparse, keep sparse (even if above threshold)
+    # apply threshold even if all sparse
     col_trans = ColumnTransformer([('trans1', OneHotEncoder(), [0]),
                                    ('trans2', OneHotEncoder(), [1])],
                                   sparse_threshold=0.2)
     res = col_trans.fit_transform(X_array)
-    assert sparse.issparse(res)
-    assert col_trans.sparse_output_
+    assert not sparse.issparse(res)
+    assert not col_trans.sparse_output_
 
     # mixed -> sparsity of (4 + 2) / 8 = 0.75
     for thres in [0.75001, 1]:
@@ -468,6 +498,17 @@ def test_column_transformer_invalid_columns(remainder):
         assert_raise_message(ValueError, "Specifying the columns",
                              ct.fit, X_array)
 
+    # transformed n_features does not match fitted n_features
+    col = [0, 1]
+    ct = ColumnTransformer([('trans', Trans(), col)], remainder=remainder)
+    ct.fit(X_array)
+    X_array_more = np.array([[0, 1, 2], [2, 4, 6], [3, 6, 9]]).T
+    ct.transform(X_array_more)  # Should accept added columns
+    X_array_fewer = np.array([[0, 1, 2], ]).T
+    err_msg = 'Number of features'
+    with pytest.raises(ValueError, match=err_msg):
+        ct.transform(X_array_fewer)
+
 
 def test_column_transformer_invalid_transformer():
 
@@ -487,29 +528,61 @@ def test_column_transformer_invalid_transformer():
 def test_make_column_transformer():
     scaler = StandardScaler()
     norm = Normalizer()
-    ct = make_column_transformer(('first', scaler), (['second'], norm))
+    ct = make_column_transformer((scaler, 'first'), (norm, ['second']))
     names, transformers, columns = zip(*ct.transformers)
     assert_equal(names, ("standardscaler", "normalizer"))
     assert_equal(transformers, (scaler, norm))
     assert_equal(columns, ('first', ['second']))
 
+    # XXX remove in v0.22
+    with pytest.warns(DeprecationWarning,
+                      match='`make_column_transformer` now expects'):
+        ct1 = make_column_transformer(([0], norm))
+    ct2 = make_column_transformer((norm, [0]))
+    X_array = np.array([[0, 1, 2], [2, 4, 6]]).T
+    assert_almost_equal(ct1.fit_transform(X_array),
+                        ct2.fit_transform(X_array))
+
+    with pytest.warns(DeprecationWarning,
+                      match='`make_column_transformer` now expects'):
+        make_column_transformer(('first', 'drop'))
+
+    with pytest.warns(DeprecationWarning,
+                      match='`make_column_transformer` now expects'):
+        make_column_transformer(('passthrough', 'passthrough'),
+                                ('first', 'drop'))
+
+
+def test_make_column_transformer_pandas():
+    pd = pytest.importorskip('pandas')
+    X_array = np.array([[0, 1, 2], [2, 4, 6]]).T
+    X_df = pd.DataFrame(X_array, columns=['first', 'second'])
+    norm = Normalizer()
+    # XXX remove in v0.22
+    with pytest.warns(DeprecationWarning,
+                      match='`make_column_transformer` now expects'):
+        ct1 = make_column_transformer((X_df.columns, norm))
+    ct2 = make_column_transformer((norm, X_df.columns))
+    assert_almost_equal(ct1.fit_transform(X_df),
+                        ct2.fit_transform(X_df))
+
 
 def test_make_column_transformer_kwargs():
     scaler = StandardScaler()
     norm = Normalizer()
-    ct = make_column_transformer(('first', scaler), (['second'], norm),
+    ct = make_column_transformer((scaler, 'first'), (norm, ['second']),
                                  n_jobs=3, remainder='drop',
-                                 sparse_threshold=0.3)
+                                 sparse_threshold=0.5)
     assert_equal(ct.transformers, make_column_transformer(
-        ('first', scaler), (['second'], norm)).transformers)
+        (scaler, 'first'), (norm, ['second'])).transformers)
     assert_equal(ct.n_jobs, 3)
     assert_equal(ct.remainder, 'drop')
-    assert_equal(ct.sparse_threshold, 0.3)
+    assert_equal(ct.sparse_threshold, 0.5)
     # invalid keyword parameters should raise an error message
     assert_raise_message(
         TypeError,
         'Unknown keyword arguments: "transformer_weights"',
-        make_column_transformer, ('first', scaler), (['second'], norm),
+        make_column_transformer, (scaler, 'first'), (norm, ['second']),
         transformer_weights={'pca': 10, 'Transf': 1}
     )
 
@@ -518,7 +591,7 @@ def test_make_column_transformer_remainder_transformer():
     scaler = StandardScaler()
     norm = Normalizer()
     remainder = StandardScaler()
-    ct = make_column_transformer(('first', scaler), (['second'], norm),
+    ct = make_column_transformer((scaler, 'first'), (norm, ['second']),
                                  remainder=remainder)
     assert ct.remainder == remainder
 
@@ -567,11 +640,11 @@ def test_column_transformer_named_estimators():
                             ('trans2', StandardScaler(with_std=False), [1])])
     assert_false(hasattr(ct, 'transformers_'))
     ct.fit(X_array)
-    assert_true(hasattr(ct, 'transformers_'))
-    assert_true(isinstance(ct.named_transformers_['trans1'], StandardScaler))
-    assert_true(isinstance(ct.named_transformers_.trans1, StandardScaler))
-    assert_true(isinstance(ct.named_transformers_['trans2'], StandardScaler))
-    assert_true(isinstance(ct.named_transformers_.trans2, StandardScaler))
+    assert hasattr(ct, 'transformers_')
+    assert isinstance(ct.named_transformers_['trans1'], StandardScaler)
+    assert isinstance(ct.named_transformers_.trans1, StandardScaler)
+    assert isinstance(ct.named_transformers_['trans2'], StandardScaler)
+    assert isinstance(ct.named_transformers_.trans2, StandardScaler)
     assert_false(ct.named_transformers_.trans2.with_std)
     # check it are fitted transformers
     assert_equal(ct.named_transformers_.trans1.mean_, 1.)
@@ -583,12 +656,12 @@ def test_column_transformer_cloning():
     ct = ColumnTransformer([('trans', StandardScaler(), [0])])
     ct.fit(X_array)
     assert_false(hasattr(ct.transformers[0][1], 'mean_'))
-    assert_true(hasattr(ct.transformers_[0][1], 'mean_'))
+    assert hasattr(ct.transformers_[0][1], 'mean_')
 
     ct = ColumnTransformer([('trans', StandardScaler(), [0])])
     ct.fit_transform(X_array)
     assert_false(hasattr(ct.transformers[0][1], 'mean_'))
-    assert_true(hasattr(ct.transformers_[0][1], 'mean_'))
+    assert hasattr(ct.transformers_[0][1], 'mean_')
 
 
 def test_column_transformer_get_feature_names():
@@ -728,7 +801,7 @@ def test_column_transformer_remainder():
         "or estimator.", ct.fit_transform, X_array)
 
     # check default for make_column_transformer
-    ct = make_column_transformer(([0], Trans()))
+    ct = make_column_transformer((Trans(), [0]))
     assert ct.remainder == 'drop'
 
 
@@ -959,3 +1032,52 @@ def test_column_transformer_callable_specifier():
     assert_array_equal(ct.fit(X_df).transform(X_df), X_res_first)
     assert callable(ct.transformers[0][2])
     assert ct.transformers_[0][2] == ['first']
+
+
+def test_column_transformer_negative_column_indexes():
+    X = np.random.randn(2, 2)
+    X_categories = np.array([[1], [2]])
+    X = np.concatenate([X, X_categories], axis=1)
+
+    ohe = OneHotEncoder(categories='auto')
+
+    tf_1 = ColumnTransformer([('ohe', ohe, [-1])], remainder='passthrough')
+    tf_2 = ColumnTransformer([('ohe', ohe,  [2])], remainder='passthrough')
+    assert_array_equal(tf_1.fit_transform(X), tf_2.fit_transform(X))
+
+
+@pytest.mark.parametrize("explicit_colname", ['first', 'second'])
+def test_column_transformer_reordered_column_names_remainder(explicit_colname):
+    """Regression test for issue #14223: 'Named col indexing fails with
+       ColumnTransformer remainder on changing DataFrame column ordering'
+
+       Should raise error on changed order combined with remainder.
+       Should allow for added columns in `transform` input DataFrame
+       as long as all preceding columns match.
+    """
+    pd = pytest.importorskip('pandas')
+
+    X_fit_array = np.array([[0, 1, 2], [2, 4, 6]]).T
+    X_fit_df = pd.DataFrame(X_fit_array, columns=['first', 'second'])
+
+    X_trans_array = np.array([[2, 4, 6], [0, 1, 2]]).T
+    X_trans_df = pd.DataFrame(X_trans_array, columns=['second', 'first'])
+
+    tf = ColumnTransformer([('bycol', Trans(), explicit_colname)],
+                           remainder=Trans())
+
+    tf.fit(X_fit_df)
+    err_msg = 'Column ordering must be equal'
+    with pytest.raises(ValueError, match=err_msg):
+        tf.transform(X_trans_df)
+
+    # No error for added columns if ordering is identical
+    X_extended_df = X_fit_df.copy()
+    X_extended_df['third'] = [3, 6, 9]
+    tf.transform(X_extended_df)  # No error should be raised
+
+    # No 'columns' AttributeError when transform input is a numpy array
+    X_array = X_fit_array.copy()
+    err_msg = 'Specifying the columns'
+    with pytest.raises(ValueError, match=err_msg):
+        tf.transform(X_array)
