@@ -95,7 +95,7 @@ def load_svmlight_file(f, n_features=None, dtype=np.float64,
 
     multilabel : boolean, optional, default False
         Samples may have several labels each (see
-        http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
+        https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
 
     zero_based : boolean or "auto", optional, default "auto"
         Whether column indices in f are zero-based (True) or one-based
@@ -134,14 +134,14 @@ def load_svmlight_file(f, n_features=None, dtype=np.float64,
 
     See also
     --------
-    load_svmlight_files: similar function for loading multiple files in this
-    format, enforcing the same number of features/columns on all of them.
+    load_svmlight_files : similar function for loading multiple files in this
+        format, enforcing the same number of features/columns on all of them.
 
     Examples
     --------
     To use joblib.Memory to cache the svmlight file::
 
-        from sklearn.utils import Memory
+        from joblib import Memory
         from sklearn.datasets import load_svmlight_file
         mem = Memory("./mycache")
 
@@ -239,7 +239,7 @@ def load_svmlight_files(files, n_features=None, dtype=np.float64,
 
     multilabel : boolean, optional
         Samples may have several labels each (see
-        http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
+        https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
 
     zero_based : boolean or "auto", optional
         Whether column indices in f are zero-based (True) or one-based
@@ -426,7 +426,7 @@ def dump_svmlight_file(X, y, f,  zero_based=True, comment=None, query_id=None,
 
     multilabel : boolean, optional
         Samples may have several labels each (see
-        http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
+        https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
 
         .. versionadded:: 0.17
            parameter *multilabel* to support multilabel datasets.
